import { Temporal } from "temporal-polyfill"
import ky from "ky"

const api = ky.extend({
	prefixUrl: "https://ias-tea-axum.shuttleapp.rs/",
	mode: "cors"
})

export interface TypeOfTea {
	id: string
	name: string
	notes: string
	waterTemperatureInCelsius: number
	steepingTime: Temporal.Duration
	registrationTimestamp: Temporal.Instant
}

interface WireTypeOfTea {
	id: number
	tea_name: string
	tea_notes: string
	water_temp: number // Celsius
	steeping_seconds: number
	registration_timestamp: string
}

export interface SteepingTimeChange {
	id: string
	typeOfTeaId: string
	timestamp: Temporal.Instant
	newValue: Temporal.Duration
}

export interface BrewingEvent {
	id: string
	typeOfTeaId: string
	timestamp: Temporal.Instant
}

interface WireBrewingEvent {
	change_id: string
	tea_id: number
	steeping_seconds: number
	steeping_tested_time: string
}

export interface Configuration {
	defaultSteepingTime: Temporal.Duration
	feedbackTimeout: Temporal.Duration
}

interface WireConfiguration {
	default_steeping_time: number // seconds
	feedback_timeout: number // seconds
}

export async function fetchConfiguration(): Promise<Configuration> {
	const result = await api.get("configuration").json<WireConfiguration>()

	return {
		defaultSteepingTime: Temporal.Duration.from({ seconds: result.default_steeping_time }).round({ largestUnit: "minutes" }),
		feedbackTimeout: Temporal.Duration.from({ seconds: result.feedback_timeout }).round({ largestUnit: "hour" }),
	}
}

export async function updateConfiguration(data: Configuration) {
	const wireData: WireConfiguration = {
		default_steeping_time: data.defaultSteepingTime.total("seconds"),
		feedback_timeout: data.feedbackTimeout.total("seconds")
	}

	await api.put("configuration", { json: wireData })
}

export async function fetchBrewingEvents(): Promise<BrewingEvent[]> {
	const result = await api.get("brewing-events").json<WireBrewingEvent[]>()
	return result.map(e => ({
		id: e.change_id,
		typeOfTeaId: e.tea_id.toString(),
		timestamp: Temporal.PlainDateTime.from(e.steeping_tested_time).toZonedDateTime("UTC").toInstant()
	}))
}

export async function fetchTypesOfTeaById(): Promise<Map<string, TypeOfTea>> {
	const result = await api.get("types-of-tea").json<WireTypeOfTea[]>()

	result.sort((a, b) => a.id - b.id)

	return new Map(result.map(t => [t.id.toString(), {
		id: t.id.toString(),
		name: t.tea_name,
		notes: t.tea_notes,
		waterTemperatureInCelsius: t.water_temp,
		steepingTime: Temporal.Duration.from({ seconds: t.steeping_seconds }).round({ largestUnit: "minutes" }), // round for balancing
		registrationTimestamp: Temporal.PlainDateTime.from(t.registration_timestamp).toZonedDateTime("UTC").toInstant()
	}]))
}

export async function updateTypeOfTea(id: string, data: Pick<TypeOfTea, "name" | "notes" | "waterTemperatureInCelsius" | "steepingTime">) {
	const wireData: Pick<WireTypeOfTea, "tea_name" | "tea_notes" | "water_temp" | "steeping_seconds"> = {
		tea_name: data.name,
		tea_notes: data.notes,
		water_temp: data.waterTemperatureInCelsius,
		steeping_seconds: data.steepingTime.total("seconds")
	}

	await api.put(`types-of-tea/${id}`, { json: wireData })
}